import django
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

SETTINGS_TYPES = {
    'POWCAPTCHA_API_URL': str,
    'POWCAPTCHA_API_KEY': str,
}

# Validate settings types.
for variable, instance_type in SETTINGS_TYPES.items():
    if hasattr(settings, variable) and not isinstance(
        getattr(settings, variable), instance_type
    ):
        raise ImproperlyConfigured(
            'Setting %s is not of type' % variable, instance_type
        )

if django.VERSION < (3, 2):
    default_app_config = 'django_recaptcha.apps.DjangoRecaptchaConfig'



import json
from urllib.error import HTTPError
from urllib.request import Request, build_opener

from django.conf import settings


class PowCaptchaValidationException(Exception):
    pass


def powcaptcha_request(path, params):
    domain = getattr(settings, 'POWCAPTCHA_API_URL')
    token = getattr(settings, 'POWCAPTCHA_API_TOKEN')

    request_object = Request(
        url=f'https://{domain}/{path}',
        data=params,
        headers={
            'Authorization': f'Bearer {token}',
        },
    )

    opener_args = []
    opener = build_opener(*opener_args)

    return opener.open(
        request_object,
        timeout=5
    )


def get_challenge():
    path = 'GetChallenges?difficultyLevel=5'
    response = powcaptcha_request(path, [])
    challenges = json.loads(response.read().decode('utf-8'))

    # @Todo: implement caching

    return challenges[0]


def validate_captcha(challenge: str, nonce: str):
    path = f'Verify?challenge={challenge}&nonce={nonce}'

    try:
        powcaptcha_request(path, [])
    except HTTPError as error:
        raise PowCaptchaValidationException(error.code)



import logging

from django import forms
from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from django_powcaptcha import client
from django_powcaptcha.client import PowCaptchaValidationException
from django_powcaptcha.widgets import PowCaptchaWidget

logger = logging.getLogger(__name__)


class PowCaptchaField(forms.CharField):
    widget = PowCaptchaWidget
    default_error_messages = {
        'captcha_invalid': _('Error verifying reCAPTCHA, please try again.'),
        'captcha_error': _('Error verifying reCAPTCHA, please try again.'),
    }

    def __init__(self, api_key=None, api_url=None, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.required = True

        # Setup instance variables.
        self.api_key = api_key or getattr(settings, 'POWCAPTCHA_API_KEY')
        self.api_url = api_url or getattr(settings, 'POWCAPTCHA_API_URL')

    def validate(self, value):
        super().validate(value)

        try:
            client.validate_captcha(
                challenge='',
                nonce='',
            )
        except PowCaptchaValidationException:
            raise ValidationError(
                self.error_messages['captcha_error'], code='captcha_error'
            )



from setuptools import find_packages, setup

long_desc = open("README.md").read()

setup(
    name="django-powcaptcha",
    version="0.0.1",
    description="Django PowCaptcha form field/widget app.",
    long_description=long_desc,
    long_description_content_type="text/markdown",
    author="Jean-Philippe Bidegain",
    author_email="jp@bidega.in",
    license="BSD",
    url="https://github.com/aeyoll/django-powcaptcha",
    project_urls={
        "Changelog": "https://github.com/aeyoll/django-powcaptcha/blob/main/CHANGELOG.md",
        "Issue Tracker": "https://github.com/aeyoll/django-powcaptcha/issues",
    },
    packages=find_packages(),
    install_requires=["django"],
    keywords=["django", "powcaptcha"],
    include_package_data=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Django",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.1",
        "Framework :: Django :: 4.2",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    zip_safe=False,
)



from django.conf import settings
from django.forms import widgets

from django_powcaptcha.client import get_challenge


class PowCaptchaWidget(widgets.Widget):
    input_type = 'hidden'
    template_name = 'django_recaptcha/widget.html'

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context.update(
            {
                'captcha_url': settings.POWCAPTCHA_API_URL,
                'captcha_challenge': get_challenge(),
                'captcha_callback': 'myCaptchaCallback',
            }
        )
        return context

    # def build_attrs(self, base_attrs, extra_attrs=None):
    #     attrs = super().build_attrs(base_attrs, extra_attrs)
    #     attrs['data-sqr-captcha-url'] = settings.POWCAPTCHA_API_URL
    #     attrs['data-sqr-captcha-challenge'] = self.get_challenge()
    #     attrs['data-sqr-captcha-callback'] = 'myCaptchaCallback'
    #     return attrs
